mod aes;
mod square;
mod utils;
use crate::aes::Aes;

use clap::{Parser, Subcommand};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
#[command(propagate_version = true)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// Ecnrypt
    Encrypt {
        /// (default Some('11'))
        #[arg(long = "nturn", short = 'n')]
        nturn: Option<String>,
        /// (default "")
        #[arg(long = "text", short = 't')]
        text: String,
        /// (default True)
        #[arg(long = "hextext", short = 'T')]
        hextext: bool,
        /// (default "")
        #[arg(long = "key", short = 'k')]
        key: String,
        /// (default True)
        #[arg(long = "hexkey", short = 'K')]
        hexkey: bool,
        /// (default False)
        #[arg(long = "decrypt", short = 'd')]
        decrypt: bool,
    },
    Findkey {
        /// (default "")
        key: String,
        /// (default True)
        #[arg(long = "hexkey", short = 'K')]
        hexkey: bool,
    },
    Square {
        /// (default "1")
        number: String,
    },
}

fn main() -> std::io::Result<()> {
    let cli = Cli::parse();

    match &cli.command {
        Commands::Encrypt {
            nturn,
            text,
            hextext,
            key,
            hexkey,
            decrypt,
        } => {
            println!("original text: {}", &text);
            let aeskey: [u8; 16];
            let result: [u8; 16];
            let aestext: [u8; 16];
            let mut aesnturn: usize = 10;

            match hexkey {
                false => aeskey = utils::parse_hex_string(&key),
                true => aeskey = utils::string_to_u8_16(&key),
            }
            match nturn {
                Some(number) => aesnturn = number.parse::<usize>().unwrap(),
                _ => {}
            }

            let aescipher = Aes::new(&aeskey, &aesnturn);

            match hextext {
                true => aestext = utils::parse_hex_string(&text),
                false => aestext = utils::string_to_u8_16(&text),
            }

            match decrypt {
                true => {
                    result = aescipher.decrypt_block(&aestext);
                    print!("Decrypted text, hex-encoded: ");
                }
                false => {
                    result = aescipher.encrypt_block(&aestext);
                    print!("Encrypted text, hex-encoded: ");
                }
            }
            Aes::print_key(&result);
            println!();
        }
        Commands::Findkey { key, hexkey } => {
            let aeskey: [u8; 16];
            match hexkey {
                false => aeskey = utils::parse_hex_string(&key),
                true => aeskey = utils::string_to_u8_16(&key),
            }
            let keyfound: [u8; 16] = Aes::findkey(&aeskey);
            if keyfound == aeskey {
                println!("key found.");
            } else {
                println!("key not found.");
            }
        }
        Commands::Square { number } => {
            Aes::test_square_attack(number.parse::<usize>().unwrap());
        }
    }
    Ok(())
}
